/*
 * Created on Sep 13, 2005
 *
 * Generic class to represent any field...
 */
package com.ibm.jps.uxtoweb.metamodel;

import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Property;

/**
 * @author jpschoch
 */
public class Field {
    /**
     * The field <code>name</code>
     */
    private String name = null;

    /**
     * <code>beanQualifiedName</code> is the qualified name of the bean class containing this field (e.g.
     * com.ibm.jps.beans.User). Note that beanQualifiedName is actually derived from bean below.
     */
    private String beanQualifiedName = "";

    /**
     * The name of the column table this field might be mapped to
     */
    private String columnName = null;
    
    /**
     * <code>bean</code> is the bean class containing this field (e.g. the field 'userName' could
     * be contained in bean class 'com.ibm.jps.beans.User'.
     */
    private Class bean = null;

    /**
     */
    private Property beanAtt = null;
    
    /**
     * <code>label</code> is used to display the filed on a screen. It defaults to the field name.
     */
    private String label = null;
    
    /**
     * mappedToTable and mappedToColumn are simple names like "BOOKS" and "AUTHOR"
     */
    private String mappedToTable = null;
    private String mappedToColumn = null;

    /**
     * A field in a list may also act as a link.
     */
    private Link link = null;

    /**
     * <code>sortable</code> applies only to fields included in a JSPList.
     */
    private boolean sortable = false;

    private boolean primitiveType = false;

    private boolean primaryKey = false;
    private boolean generateKey = false;
    
    /**
     * @return Returns the label.
     */
    public String getLabel() {
        return label;
    }

    /**
     * @param label
     *            The label to set.
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * @param name
     * @param type
     */
    public Field(String name, String type) {
        super();
        this.name = name;
        this.beanQualifiedName = type;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }

    /**
     * @param name
     *            The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return Returns the type.
     */
    public String getBeanQualifiedName() {
        return beanQualifiedName;
    }

    /**
     * @param type
     *            The type to set.
     */
    public void setBeanQualifiedName(String type) {
        this.beanQualifiedName = type;
    }

    public boolean isPrimitiveType() {
        return primitiveType;
    }

    public void setPrimitiveType(boolean primitiveType) {
        this.primitiveType = primitiveType;
    }

    public boolean isSortable() {
        return sortable;
    }

    public void setSortable(boolean sortable) {
        this.sortable = sortable;
    }

    public Link getLink() {
        return link;
    }

    public void setLink(Link link) {
        this.link = link;
    }

    public boolean isPrimaryKey() {
        return primaryKey;
    }

    public void setPrimaryKey(boolean primaryKey) {
        this.primaryKey = primaryKey;
    }

    public Class getBean() {
        return bean;
    }

    public void setBean(Class bean) {
        this.bean = bean;
    }
    public Property getBeanAtt() {
        return beanAtt;
    }
    public void setBeanAtt(Property beanAtt) {
        this.beanAtt = beanAtt;
    }
    public String getColumnName() {
        return columnName;
    }
    public void setColumnName(String columnName) {
        this.columnName = columnName;
    }
    public String getMappedToColumn() {
        return mappedToColumn;
    }
    public void setMappedToColumn(String mappedToColumn) {
        this.mappedToColumn = mappedToColumn;
    }
    public String getMappedToTable() {
        return mappedToTable;
    }
    public void setMappedToTable(String mappedToTable) {
        this.mappedToTable = mappedToTable;
    }
    public boolean isGenerateKey() {
        return generateKey;
    }
    public void setGenerateKey(boolean generateKey) {
        this.generateKey = generateKey;
    }
}
