/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2003, 2004. All Rights Reserved.               |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.jps.uxtoweb.transformation;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.UMLPackage;

import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.core.ITransformationDescriptor;
import com.ibm.xtools.transform.core.RootTransform;

/**
 * The root transform for the UXToStruts transformation that converts UX classes
 * to Struts files.
 */
public class UXToWebRootTransform extends RootTransform {

    public static final String ID = "com.ibm.jps.uxtoweb.transformation"; //$NON-NLS-1$

    /**
     * Constructor.
     * 
     * @param descriptor
     *            A transformation descriptor.
     */
    public UXToWebRootTransform(ITransformationDescriptor descriptor) {
        super(descriptor);

        // Initialize root transform with main UML transform.
        // The false argument indicates the main transform will only
        // accept individual source objects (and not the list of selected
        // objects).

        UXToWebTransform mainTransform = new UXToWebTransform(descriptor.getId());
        mainTransform.setName(descriptor.getName());

        setupInitialize();
        initialize(mainTransform, false);
        setupFinalize();
    }

    /**
     * Add rules in this method that will be called once before the main UML
     * transform is executed.
     */
    private void setupInitialize() {
    }

    /**
     * Add rules in this method that will be called once after the main UML
     * transform has been executed.
     */
    private void setupFinalize() {
    }

    /**
     * Determines if any member in the list of source objects specified in the
     * context is acceptable. This transformation accepts a model element of
     * type <code>class</code>.
     * 
     * This method iterates over the list supplied as the source and returns
     * true if any of the members is a UML2 class.
     * 
     * @param context
     *            A context that contains the source and target info.
     * @return boolean - true if the context is acceptable
     */
    public boolean canAccept(ITransformContext context) {
        boolean accepted = false;
        UMLPackage uml = UMLPackage.eINSTANCE;
        if (context.getSource() instanceof List) {
            Iterator iter = ((List) context.getSource()).iterator();
            while (!accepted && iter.hasNext()) {
                Object element = iter.next();
                if (element instanceof EObject) {
                    EClass kind = ((EObject) element).eClass();
                    accepted = (kind == uml.getClass_()) || (kind == uml.getPackage())
                            || (kind == uml.getModel());
                }
            }
        }
        return accepted;
    }
}
