/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2003, 2004. All Rights Reserved.               |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.jps.uxtoweb.transformation;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IStatus;

import com.ibm.jps.uxtoweb.utility.StatusUtil;
import com.ibm.xtools.transform.core.AbstractTransform;
import com.ibm.xtools.transform.core.AbstractTransformationProvider;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.core.ITransformationDescriptor;

/**
 * UXToWebTransformationProvider is registered in plugin.xml as a Transformation
 * Provider.
 */
/**
 * @see com.ibm.xtools.transform.core.AbstractTransformationProvider
 */
public class UXToWebTransformationProvider extends
		AbstractTransformationProvider {

	/**
	 * @see com.ibm.xtools.transform.core.AbstractTransformationProvider#createTransformation(com.ibm.xtools.transform.core.ITransformationDescriptor)
	 */
	public AbstractTransform createTransformation(
			ITransformationDescriptor descriptor) {
		if (descriptor.getId().startsWith(UXToWebRootTransform.ID)) {
			UXToWebRootTransform transform = new UXToWebRootTransform(
					descriptor);
			return transform;
		} else
			return null;
	}

	/**
	 * @see com.ibm.xtools.transform.core.AbstractTransformationProvider#validateContext(com.ibm.xtools.transform.core.ITransformationDescriptor,
	 *      com.ibm.xtools.transform.core.ITransformContext)
	 */
	public IStatus validateContext(ITransformationDescriptor descriptor,
			ITransformContext context) {
		AbstractTransform transform = context.getTransform();
		boolean isSourceOk = transform.canAccept(context);
		Object target = context.getTargetContainer();
		boolean isTargetOk = (target instanceof IProject) ? true : false;
		// TODO should check for JSF vs. Struts
		boolean arePropertiesOk = true;

		final String sourceErrMsg = "Source invalide"; //$NON-NLS-1$
		final String targetErrMsg = "Cible invalide"; //$NON-NLS-1$
		// final String sourceErrMsg = ResourceManager
		// .getString("UXToWebTransformation.invalidSourceMsg"); //$NON-NLS-1$
		// final String targetErrMsg = ResourceManager
		// .getString("UXToWebTransformation.invalidTargetMsg"); //$NON-NLS-1$
		final String pluginId = "com.ibm.jps.uxtoweb";

		return StatusUtil.createTransformContextValidationStatus(pluginId,
				isSourceOk, sourceErrMsg, isTargetOk, targetErrMsg,
				arePropertiesOk);
	}

}
