/*
 * Created on Sep 16, 2005
 */
package com.ibm.jps.uxtoweb.transformation.rules;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;

import com.ibm.jps.uxtoweb.metamodel.AppData;
import com.ibm.jps.uxtoweb.metamodel.Bean;
import com.ibm.jps.uxtoweb.metamodel.DataBase;
import com.ibm.jps.uxtoweb.metamodel.Image;
import com.ibm.jps.uxtoweb.metamodel.Jsp;
import com.ibm.jps.uxtoweb.metamodel.UserAction;
import com.ibm.jps.uxtoweb.utility.JavaUtil;
import com.ibm.jps.uxtoweb.utility.ResourceUtil;
import com.ibm.xtools.transform.core.AbstractRule;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.uml.transform.core.IsElementKindCondition;

/**
 * @author jpschoch
 */
public class CodeGenRule extends AbstractRule {
    public final static String ID = "com.ibm.jps.uxtoweb.transformation.saveprojectRule"; //$NON-NLS-1$
    public final static String NAME = "UXToWeb.saveProjectRule.name"; //$NON-NLS-1$

    // TODO need an XML based config file to determine what to build

    /**
     *  
     */
    public CodeGenRule() {
        super();
        setAcceptCondition(new IsElementKindCondition(UMLPackage.eINSTANCE.getModel()));
    }

    /**
     * @param id
     * @param name
     */
    public CodeGenRule(String id, String name) {
        super(id, name);
        setAcceptCondition(new IsElementKindCondition(UMLPackage.eINSTANCE.getModel()));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.xtools.transform.core.AbstractRule#createTarget(com.ibm.xtools.transform.core.ITransformContext)
     */
    protected Object createTarget(ITransformContext context) throws Exception {
        boolean validated = ((Boolean) context.getPropertyValue("ModelValidated")).booleanValue(); //$NON-NLS-1$
        if (!validated) return null;
        // Transformation specific label
        String tLabel = (context.getTransform().getId().endsWith("struts") ? "struts" : "jsf");
        System.out.println("--- Code generation rule");
        NamedElement element = (NamedElement) context.getSource();
        Object targetContainer = context.getTargetContainer();
        IProject project = (targetContainer instanceof IProject) ? (IProject) targetContainer : ResourceUtil
                .getProject(element);
        AppData app = (AppData) context.getPropertyValue("AppData"); //$NON-NLS-1$
        DataBase db = app.getDataBase();

        // Add necessary libraries 
        List<String> additionalLib = new ArrayList<String>();
        if (!ResourceUtil.isLibraryIncluded("jsf-ibm.jar", project)) additionalLib.add("jsf-ibm.jar");
        if (db != null) {
            if (!ResourceUtil.isLibraryIncluded("antlr.jar", project)) additionalLib.add("antlr.jar");
            if (!ResourceUtil.isLibraryIncluded("asm.jar", project)) additionalLib.add("asm.jar");
            if (!ResourceUtil.isLibraryIncluded("asm-attrs.jar", project)) additionalLib.add("asm-attrs.jar");
            if (!ResourceUtil.isLibraryIncluded("cglib-2.1.3.jar", project)) additionalLib.add("cglib-2.1.3.jar");
            if (!ResourceUtil.isLibraryIncluded("commons-collections-2.1.1.jar", project)) additionalLib.add("commons-collections-2.1.1.jar");
            if (!ResourceUtil.isLibraryIncluded("commons-logging-1.0.4.jar", project)) additionalLib.add("commons-logging-1.0.4.jar");
            if (!ResourceUtil.isLibraryIncluded("dom4j-1.6.1.jar", project)) additionalLib.add("dom4j-1.6.1.jar");
            if (!ResourceUtil.isLibraryIncluded("hibernate3.jar", project)) additionalLib.add("hibernate3.jar");
            if (!ResourceUtil.isLibraryIncluded("jta.jar", project)) additionalLib.add("jta.jar");            
            if (!ResourceUtil.isLibraryIncluded("db2j.jar", project)) additionalLib.add("db2j.jar");            
        }
        if (additionalLib.size() > 0) {
            ResourceUtil.addLibrariesToProject(additionalLib, "lib", project, "WebContent" + File.separatorChar + "WEB-INF" + File.separatorChar + "lib");
        }
        
        //Java Utilities
        JETEmitter utilBuilder = ResourceUtil.buildEmitter("templates/java/JavaUtil.javajet");
        ResourceUtil.generate(null, utilBuilder, project,
                "src" + File.separatorChar + "uxtoweb" + File.separatorChar + "util", "JavaUtil.java", true); //$NON-NLS-1$ //$NON-NLS-2$

        // Complete
		IProject jetProject = ResourcesPlugin.getWorkspace().getRoot().getProject(utilBuilder.getProjectName());
		if (jetProject != null) {
			System.out.println(">>> Adding java files");
	        additionalLib = new ArrayList<String>();
	        additionalLib.add("AppData.java");
	        additionalLib.add("Bean.java");
	        additionalLib.add("DataBase.java");
	        additionalLib.add("Field.java");
	        additionalLib.add("Forward.java");
	        additionalLib.add("Image.java");
	        additionalLib.add("ItemList.java");
	        additionalLib.add("Jsp.java");
	        additionalLib.add("Link.java");
	        additionalLib.add("ListBox.java");
	        additionalLib.add("Method.java");
	        additionalLib.add("RadioButton.java");
	        additionalLib.add("TextField.java");
	        additionalLib.add("UserAction.java");
	        additionalLib.add("UserActionKind.java");
	        ResourceUtil.addFilesToProject(additionalLib, "src\\com\\ibm\\jps\\uxtoweb\\metamodel", jetProject, "src\\com\\ibm\\jps\\uxtoweb\\metamodel");
	        additionalLib = new ArrayList<String>();
	        additionalLib.add("JavaUtil.java");
	        ResourceUtil.addFilesToProject(additionalLib, "src\\com\\ibm\\jps\\uxtoweb\\utility", jetProject, "src\\com\\ibm\\jps\\uxtoweb\\utility");
		}

		// Generate configuration files
        if (app != null) {
            JETEmitter configBuilder = ResourceUtil.buildEmitter("templates/" + tLabel + "/config/config.javajet");
            String filePrefix = (tLabel.equals("jsf") ? "faces" : "struts");
            ResourceUtil.generate(app, configBuilder, project, "WebContent" + File.separatorChar + "WEB-INF",
                    filePrefix + "-config.xml", true);
        }

        // Logo file
        Image logo = app.getLogo();
        if (logo != null) {
            System.out.println("    Copying logo file " + logo.getFPath() + File.separatorChar + logo.getFName());
            ResourceUtil.copyToProjectFile(logo.getFPath() + File.separatorChar + logo.getFName(), project,
                    "WebContent" + File.separatorChar + "theme", logo.getFName(), true); //$NON-NLS-1$ //$NON-NLS-2$
        }

        // JSPs
        JETEmitter jspBuilder = ResourceUtil.buildEmitter("templates/" + tLabel + "/jsp/jsp.javajet");
        JETEmitter facesControllerBuilder = ResourceUtil.buildEmitter("templates/jsf/jsp/FacesController.javajet");
        for (Iterator i = app.getAppJSPs().iterator(); i.hasNext();) {
            Jsp jsp = (Jsp) i.next();
            ResourceUtil.generate(jsp, jspBuilder, project, "WebContent", jsp.getName() + ".jsp", true);
            if (tLabel.equals("jsf")) {
                String path = jsp.getSrcPath().replace('.', File.separatorChar) + File.separatorChar + "control";
                ResourceUtil.generate(jsp, facesControllerBuilder, project, "src" + File.separatorChar + path,
                        JavaUtil.firstCharToUpperCase(jsp.getName()) + ".java", false);
            }
        }

        // User actions: for Struts only. (For JSF, processing of useractions included in bean
        // processing.)
        if (tLabel.equals("struts")) {
            JETEmitter actionBuilder = ResourceUtil.buildEmitter("templates/struts/actions/ActionBuilder.javajet");
            for (Iterator i = app.getUserActions().iterator(); i.hasNext();) {
                UserAction action = (UserAction) i.next();
                String path = JavaUtil.getPath(action.getQualifiedName()).replace('.', File.separatorChar);
                ResourceUtil.generate(action, actionBuilder, project, "src" + File.separatorChar + path,
                        JavaUtil.getSimpleName(action.getQualifiedName()) + ".java", false); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }

        // Beans
        // TODO unify the Struts and JSF processing
        if (tLabel.equals("struts")) {
            JETEmitter beanBuilder = ResourceUtil.buildEmitter("templates/struts/beans/BeanBuilder.javajet");
            for (Iterator i = app.getBeans().iterator(); i.hasNext();) {
                Bean bean = (Bean) i.next();
                String path = JavaUtil.getPath(bean.getQualifiedName()).replace('.', File.separatorChar);
                ResourceUtil.generate(bean, beanBuilder, project, "src" + File.separatorChar + path, JavaUtil
                        .firstCharToUpperCase(JavaUtil.getSimpleName(bean.getQualifiedName()))
                        + ".java", false); //$NON-NLS-1$ //$NON-NLS-2$
            }
        } else if (tLabel.equals("jsf")) {
            JETEmitter beanManager = ResourceUtil.buildEmitter("templates/jsf/beans/BeanManager.javajet");
            for (Iterator i = app.getBeans().iterator(); i.hasNext();) {
                Bean bean = (Bean) i.next();
                ResourceUtil.generate(bean, beanManager, project, "src" + File.separatorChar + "uxtoweb"
                        + File.separatorChar + "control", JavaUtil.firstCharToUpperCase(JavaUtil.getSimpleName(bean
                        .getQualifiedName()))
                        + "Manager.java", true); //$NON-NLS-1$ //$NON-NLS-2$
            }
            JETEmitter cssBuilder = ResourceUtil.buildEmitter("templates/jsf/jsp/stylesheet.javajet");
            ResourceUtil.generate(null, cssBuilder, project,
                    "WebContent" + File.separatorChar + "theme", "stylesheet.css", true); //$NON-NLS-1$ //$NON-NLS-2$
        }

        // Standard classes
        if (tLabel.equals("jsf")) {
            JETEmitter javaBuilder = ResourceUtil.buildEmitter("templates/java/JavaClassBuilder.javajet");
            for (Iterator i = app.getClasses().iterator(); i.hasNext();) {
                Class cl = (Class) i.next();
                String path = "src" + File.separatorChar
                        + JavaUtil.convertToJavaPath(cl.getQualifiedName()).replace('.', File.separatorChar);
                String fileName = JavaUtil.firstCharToUpperCase(JavaUtil
                        .convertToSimpleJavaName(cl.getQualifiedName()))
                        + ".java";
                ResourceUtil.generate(cl, javaBuilder, project, path, fileName, true);
            }
        }

        // Hibernate
        if (db != null) {
            JETEmitter hConfigBuilder = ResourceUtil.buildEmitter("templates/hibernate/hibernate-config.javajet");
            JETEmitter hMapBuilder = ResourceUtil.buildEmitter("templates/hibernate/hibernate-table-map.javajet");
            JETEmitter hUtilBuilder = ResourceUtil.buildEmitter("templates/hibernate/hibernate-util.javajet");

            // Build the Hibernate configuration file
            ResourceUtil.generate(app, hConfigBuilder, project, "src", "hibernate.cfg.xml", true); //$NON-NLS-1$ //$NON-NLS-2$

            // Build the persistent mapping table
            for (Iterator i = app.getBeans().iterator(); i.hasNext();) {
                Bean bean = (Bean) i.next();
                if (bean.getMappedToTable() != null && !bean.getMappedToTable().equals("")) {
                    String path = JavaUtil.getPath(bean.getQualifiedName()).replace('.', File.separatorChar);
                    ResourceUtil
                            .generate(bean, hMapBuilder, project, "src" + File.separatorChar + path, JavaUtil
                                    .firstCharToUpperCase(JavaUtil.getSimpleName(bean.getQualifiedName()))
                                    + ".hbm.xml", true); //$NON-NLS-1$
                }
            }

            // Build the Hibernate Utility class
            ResourceUtil.generate(null, hUtilBuilder, project,
                    "src" + File.separatorChar + "uxtoweb" + File.separatorChar + "util", "HbmUtil.java", true); //$NON-NLS-1$ //$NON-NLS-2$
        }

        return null;
    }
}