/*
 * Created on Sep 16, 2005
 */
package com.ibm.jps.uxtoweb.transformation.rules;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;

import com.ibm.jps.uxtoweb.l18n.ResourceManager;
import com.ibm.jps.uxtoweb.metamodel.AppData;
import com.ibm.jps.uxtoweb.metamodel.Bean;
import com.ibm.jps.uxtoweb.metamodel.DataBase;
import com.ibm.jps.uxtoweb.metamodel.Field;
import com.ibm.jps.uxtoweb.metamodel.Image;
import com.ibm.jps.uxtoweb.metamodel.ItemList;
import com.ibm.jps.uxtoweb.metamodel.Jsp;
import com.ibm.jps.uxtoweb.metamodel.Link;
import com.ibm.jps.uxtoweb.metamodel.Method;
import com.ibm.jps.uxtoweb.metamodel.UserAction;
import com.ibm.jps.uxtoweb.utility.InvalidUXModelException;
import com.ibm.jps.uxtoweb.utility.JavaUtil;
import com.ibm.jps.uxtoweb.utility.ModelUtil;
import com.ibm.jps.uxtoweb.utility.ResourceUtil;
import com.ibm.xtools.transform.core.AbstractRule;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.uml.transform.core.IsElementKindCondition;

/**
 * @author jpschoch
 */
public class ExtractionRule extends AbstractRule {
    public final static String ID = "com.ibm.jps.uxtoweb.transformation.modelRule"; //$NON-NLS-1$
    public final static String NAME = ResourceManager.getString("UXToWeb.modelRule.name"); //$NON-NLS-1$
    private String appName = "";
    private Image logo = null;

    private AppData app = null;
    private List<Bean> beans = null;
    private List<UserAction> allActions = null;
    private List<Jsp> appJSPs = null;
    private List<Class> appClasses = null;
    private IResource mRsrc = null;
    // If an error is found during the analysis of a UML element, 'validated' is set to 'false',
    // which will prevent code from being generated for this element (see SaveProjectRule class).
    private boolean validated = true;

    /**
     * Default constructor. Initializes the name and the id of the rule, and initializes itself to
     * accept only a UML model as source.
     */
    public ExtractionRule() {
        super(ID, NAME);
        setAcceptCondition(new IsElementKindCondition(UMLPackage.eINSTANCE.getModel()));
    }

    /**
     * @param id
     * @param name
     */
    public ExtractionRule(String id, String name) {
        super(id, name);
        setAcceptCondition(new IsElementKindCondition(UMLPackage.eINSTANCE.getModel()));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.xtools.transform.core.AbstractRule#createTarget(com.ibm.xtools.transform.core.ITransformContext)
     */
    protected Object createTarget(ITransformContext context) throws Exception {
        // Init
        validated = true;
        NamedElement element = (NamedElement) context.getSource(); // A model
        Model model = (Model) element;
        Object targetContainer = context.getTargetContainer();
        IProject project = (targetContainer instanceof IProject) ? (IProject) targetContainer : ResourceUtil
                .getProject(element);
        System.out.println("--- Processing model " + model.getName() + "; " + project.getName());
        logo = null;
        // Marker management
        mRsrc = ModelUtil.getModelResource(model);
        if (mRsrc != null) mRsrc.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
        if (mRsrc != null) mRsrc.deleteMarkers("com.ibm.jps.uxtoweb.uxToWebMarker", true, IResource.DEPTH_INFINITE);

        // Init the object (app) that will represent the Web application in the internal meta-model
        app = new AppData();
        app.setMessageResources(project.getName().toLowerCase() + ".resources.ApplicationResources");
        beans = new ArrayList<Bean>();
        allActions = new ArrayList<UserAction>();
        appJSPs = new ArrayList<Jsp>();
        appClasses = new ArrayList<Class>();

        // Walk through all existing classes in the model
        processPackage(model);

        // Add the logo+db to every JSP (probably a better way to do this)
        for (Iterator i = appJSPs.iterator(); i.hasNext();) {
            Jsp jsp = (Jsp) i.next();
            jsp.setLogo(logo);
            jsp.setAppName(appName);
            jsp.setDataBase(app.getDataBase());
        }

        // For existing beans, check the bean fields and init the mappedToTable bean property to the
        // equivalent property in the first bean field having a non-null mappedToTable property.
        // Primary keys must be mapped to columns.
        for (Iterator i = beans.iterator(); i.hasNext();) {
            Bean b = (Bean) i.next();
            for (Iterator j = b.getFields().iterator(); j.hasNext();) {
                Field f = (Field) j.next();
                String ft = f.getMappedToTable();
                if (ft != null && !ft.equals("")) {
                    b.setMappedToTable(ft);
                    // The following statement ensures that the table associated with the primary
                    // key is retained (in case several fields are mapped to different tables.
                    if (f.isPrimaryKey()) break;
                } else if (f.isPrimaryKey()) {
                    validated = false;
                    ModelUtil
                            .addMarker(
                                    mRsrc,
                                    "Primary key "
                                            + f.getName()
                                            + " in <<bean>> class "
                                            + b.getName()
                                            + " is not mapped to a valid column. To correct this, add a dependency from the primary key to an attribute of a <<table>> class."
                                            + ".", IMarker.SEVERITY_ERROR, b.getUmlClass());
                }
            }
        }

        // Complete the app object
        app.setUserActions(allActions);
        app.setBeans(beans);
        app.setAppJSPs(appJSPs);
        for (Iterator i = appJSPs.iterator(); i.hasNext();) {
            Jsp jsp = (Jsp) i.next();
            ItemList iList = jsp.getList();
            if (iList != null) {
                for (Iterator j = iList.getFields().iterator(); j.hasNext();) {
                    Field f = (Field) j.next();
                    Link l = f.getLink();
                    if (l != null) {
                        Jsp targetJsp = findMatchingJSP(l.getTargetJSPQualifiedName());
                        if (jsp != null) {
                            List<String> targetList = targetJsp.getScriptVar();
                            String s = f.getBean().getName();
                            if (!targetList.contains(s)) {
                                targetList.add(s);
                            }
                        }
                    }
                }
            }
        }
        app.setLogo(logo);
        app.setClasses(appClasses);

        // Save the necessary data for code generation in the transformation context
        context.setPropertyValue("AppData", app);
        context.setPropertyValue("ModelValidated", new Boolean(validated));
        return null;
    }

    private void processPackage(Package pkg) {
        processOnePackage(pkg);
        for (Iterator i = pkg.getNestedPackages().iterator(); i.hasNext();) {
            processPackage((Package) i.next());
        }
    }

    private void processOnePackage(Package pkg) {
        for (Iterator i = pkg.getPackagedElements().iterator(); i.hasNext();) {
            Element elt = (Element) i.next();
            if (elt instanceof Component) {
                processComponent((Component) elt);
            } else if (elt instanceof Class) {
                processClass((Class) elt);
            }
        }
    }

    /**
     * @param component
     */
    private void processComponent(Component comp) {
        System.out.println("--- Processing component " + comp.getName());

        // <<database>> component processing
        // TODO Currently no checking for <<table>> elements. Tables are identified through their
        // mapping between table columns and bean properties.
        Stereotype st = comp.getAppliedStereotype("DataModeling::database");
        if (st != null && app.getDataBase() == null) {
            System.out.println("    ... is <<database>>");
            DataBase db = new DataBase();

            // Connection parameters
            String s = (String) comp.getValue(st, "connectionDriverClass");
            if (s.length() == 0) {
                validated = false;
                ModelUtil.addMarker(mRsrc, "Connection Driver Class not defined in <<database>> component "
                        + comp.getName() + ".", IMarker.SEVERITY_WARNING, comp);
            }
            db.setConnectionDriverClass(s);
            s = (String) comp.getValue(st, "connectionUrl");
            if (s.length() == 0) {
                validated = false;
                ModelUtil.addMarker(mRsrc, "Connection Url not defined in <<database>> component " + comp.getName()
                        + ".", IMarker.SEVERITY_WARNING, comp);
            }
            db.setConnectionUrl(s);
            s = (String) comp.getValue(st, "connectionUserName");
            db.setConnectionUserName(s);
            s = (String) comp.getValue(st, "connectionPassword");
            db.setConnectionPassword(s);
            app.setDataBase(db);
        }
    }

    private void processClass(Class cl) {
        // Init
        System.out.println("--- Processing class " + cl.getName());
        Jsp jsp = null;
        Bean bean = null;
        boolean isStandardClass = true;

        // <<screen>> class processing
        Stereotype st = cl.getAppliedStereotype("UXModeling2::screen");
        if (st != null) {
            System.out.println("    ... is <<screen>>");
            isStandardClass = false;
            checkClass(cl, st);

            // A screen is rendered as a jsp.
            jsp = new Jsp();

            // Jsp.name
            String jspName = JavaUtil.firstCharToLowerCase(cl.getName());
            jsp.setName(jspName);

            // Jsp.originalQualifiedName
            jsp.setOriginalQualifiedName(cl.getQualifiedName());

            // Jsp.title
            String jpsTitle = (String) cl.getValue(st, "label");
            if (jpsTitle == null || jpsTitle.length() == 0) jpsTitle = JavaUtil.convertToTitle(cl.getName());
            jsp.setTitle(jpsTitle);

            // Jsp.srcPath
            jsp.setSrcPath(JavaUtil.getPath(JavaUtil.replaceAllSubstrings(JavaUtil.getNameWithoutModel(cl
                    .getQualifiedName()), "::", ".")));

            // Jsp.heading and Jsp.headingLevel
            String heading = (String) cl.getValue(st, "heading");
            if (heading != null) {
                jsp.setHeading(heading);
                String hLevStr = (String) cl.getValue(st, "headingLevel");
                int hLev = (new Integer(hLevStr)).intValue();
                // TODO replace by profile constraint??
                if (hLev < 1 || hLev > 6) {
                    ModelUtil.addMarker(mRsrc, "headingLevel attribute for <<screen>> stereotype of class "
                            + cl.getName() + " outside valid range (1-6): " + hLevStr + "; defaulting to 2.",
                            IMarker.SEVERITY_WARNING, cl);
                    hLevStr = "2";
                }
                jsp.setHeadingLevel(hLevStr);
            }

            // Links and useractions
            List<Link> links = new ArrayList<Link>();
            List<UserAction> actions = new ArrayList<UserAction>();
            for (Iterator i = cl.getOwnedOperations().iterator(); i.hasNext();) {
                Operation op = (Operation) i.next();
                Stereotype opst = op.getAppliedStereotype("UXModeling2::link");
                if (opst != null) {
                    try {
                        Link link = ModelUtil.populateLink(cl, op, opst, mRsrc);
                        links.add(link);
                    } catch (InvalidUXModelException e) {
                        validated = false;
                    }
                } else {
                    opst = op.getAppliedStereotype("UXModeling2::useraction");
                    if (opst != null) {
                        UserAction action;
                        try {
                            action = ModelUtil.populateAction(cl, op, opst, mRsrc);
                            if (action != null) {
                                if (action.getMethod() == null) {
                                    validated = false;
                                    ModelUtil.addMarker(mRsrc, "Cannot find bean class for <<useraction>> "
                                            + op.getName() + " in <<screen>> class " + cl.getName()
                                            + ". To correct this, add a dependency from " + cl.getName()
                                            + " to the <<bean>> class containing the operation named " + op.getName()
                                            + ".", IMarker.SEVERITY_ERROR, cl);
                                }
                                if (action.getForwards() == null) {
                                    validated = false;
                                    ModelUtil.addMarker(mRsrc, "No forwards identified for <<useraction>> "
                                            + op.getName() + ". To correct this, add directed associations named '"
                                            + op.getName() + "' or '" + op.getName()
                                            + " [condition]' or '[condition]' from class " + cl.getName()
                                            + " to other <<screen>> classes.", IMarker.SEVERITY_ERROR, op);
                                }
                                allActions.add(action);
                                actions.add(action);
                            }
                        } catch (InvalidUXModelException e) {
                            validated = false;
                        }
                    }
                }
            }
            jsp.setLinks(links);
            jsp.setActions(actions);

            // Global forwards: correspond to associations named for instance '[success]' or
            // '[failure]'.
            jsp.setGlobalForwards(ModelUtil.findGlobalForwards(cl));

            // Init scripting variables list: this list is filled later on when all JSPs have been
            // identified
            jsp.setScriptVar(new ArrayList<String>());

            try {
                // Display fields: look for attributes with a <<display>> stereotype
                List<Field> fields = new ArrayList<Field>();
                ModelUtil.addFields(fields, cl, false, mRsrc);
                jsp.setDisplayFields(fields);

                // Input fields: look for a composition with an <<input>> class
                fields = new ArrayList<Field>();
                Class wcl = ModelUtil.findAssocClass(cl, "UXModeling2::input", AggregationKind
                        .get(AggregationKind.COMPOSITE));
                if (wcl != null) ModelUtil.addFields(fields, wcl, true, mRsrc);
                jsp.setInputFields(fields);

                // ItemList: look for a composition with a <<list>> class
                jsp.setList(ModelUtil.populateList(cl, mRsrc));

                // Add the completed JSP to the list of JSPs
                appJSPs.add(jsp);
            } catch (InvalidUXModelException e) {
                validated = false;
            }
        }

        // <<bean>> class processing
        // Note that bean classes are also generated as standard Java classes, which is why
        // isStandardClass is not set to false
        st = cl.getAppliedStereotype("UXModeling2::bean");
        if (st != null) {
            String beanQualifiedName = JavaUtil.getQualifiedName(cl, st);
            bean = new Bean(beanQualifiedName);
            System.out.println("    ... is <<bean>> (qualified name = " + beanQualifiedName + ")");
            checkClass(cl, st);
            bean.setUmlClass(cl);
            List<Field> fields = new ArrayList<Field>();
            for (Iterator i = cl.getAttributes().iterator(); i.hasNext();) {
                Property att = (Property) i.next();
                if (att.getType() != null) {
                    Field f = new Field(att.getName(), JavaUtil.replaceAllSubstrings(JavaUtil.getNameWithoutModel(att
                            .getType().getQualifiedName()), "::", "."));
                    Stereotype pkSt = att.getAppliedStereotype("DataModeling::pk");
                    if (pkSt != null) {
                        f.setPrimaryKey(true);
                        f.setGenerateKey(((Boolean) att.getValue(pkSt, "generateKey")).booleanValue());
                    }
                    ModelUtil.setMatchingColumn(att, f);
                    fields.add(f);
                }
            }
            bean.setFields(fields);
            List<Method> methods = new ArrayList<Method>();
            for (Iterator i = cl.getOwnedOperations().iterator(); i.hasNext();) {
                Operation op = (Operation) i.next();
                Method m = new Method();
                m.setName(op.getName());
                methods.add(m);
            }
            bean.setMethods(methods);
            List<String> dependencies = new ArrayList<String>();
            for (Iterator i = cl.getClientDependencies().iterator(); i.hasNext();) {
                Dependency dep = (Dependency) i.next();
                for (Iterator j = dep.getSuppliers().iterator(); j.hasNext();) {
                    NamedElement elt = (NamedElement) j.next();
                    if (elt instanceof Class) {
                        Class supplier = (Class) elt;
                        dependencies.add(JavaUtil.replaceAllSubstrings(JavaUtil.getNameWithoutModel(supplier
                                .getQualifiedName()), "::", "."));
                    }
                }
            }
            bean.setDependencies(dependencies);
            beans.add(bean);
        }

        // <<global>> class defines model-level settings. Should be unique.
        st = cl.getAppliedStereotype("UXModeling2::global");
        if (st != null) {
            System.out.println("    ... is <<global>> ");
            isStandardClass = false;
            String logoFName = (String) cl.getValue(st, "logofile");
            if (logoFName != null && !logoFName.equals("")) {
                IProject prj = ModelUtil.getProject(cl);
                IFile f = prj.getFile(logoFName);
                if (!f.exists()) {
                    logo = null;
                    ModelUtil.addMarker(mRsrc, "Invalid logo file name " + logoFName
                            + ". The logo file should be located in the same project as the model itself. Examples: '"
                            + "logofile.gif' or 'subdir\\logofile.jpg'.", IMarker.SEVERITY_ERROR, null);

                } else {
                    logo = new Image();
                    logo.setFName(JavaUtil.getSimpleName(f.getFullPath().toOSString(), File.separatorChar));
                    logo.setFPath(ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString()
                            + JavaUtil.getPath(f.getFullPath().toOSString(), File.separatorChar));
                    Integer temp = (Integer) cl.getValue(st, "logowidth");
                    if (temp != null) logo.setWidth(temp);
                    temp = (Integer) cl.getValue(st, "logoheight");
                    if (temp != null) logo.setHeight(temp);
                }
            }
            appName = (String) cl.getValue(st, "appname");
            if (appName == null) appName = "";
        }

        if (isStandardClass) {
            // Filter out classes at the root, classes with a <<list>>, <<input>> or <<table>>
            // stereotype
            if (cl.getAppliedStereotype("UXModeling2::list") == null
                    && cl.getAppliedStereotype("UXModeling2::input") == null
                    && cl.getAppliedStereotype("DataModeling::table") == null
                    && !JavaUtil.getPath(JavaUtil.getNameWithoutModel(cl.getQualifiedName())).equals("")) {
                System.out.println("    ... added to list of regular classes");
                appClasses.add(cl);
            }
        }

    }

    private void checkClass(Class cl, Stereotype st) {
        String path = "";
        // Class must not be in top level package
        if (st != null) path = (String) cl.getValue(st, "path");
        if (path == null || path.equals(""))
                path = JavaUtil.getPath(JavaUtil.replaceAllSubstrings(JavaUtil.getNameWithoutModel(cl
                        .getQualifiedName()), "::", "."));
        if (path.equals("")) {
            validated = false;
            ModelUtil
                    .addMarker(
                            mRsrc,
                            "Cannot generate Java for class "
                                    + cl.getName()
                                    + " as it is located in the root package of the model. To correct this, move the class to a package in the model or change the 'path' attribute of its UXModeling stereotype (if applicable) to a non-empty value.",
                            IMarker.SEVERITY_ERROR, cl);
        }
        // Class must not be in a <<perspective>> package
        Package p = cl.getNearestPackage();
        while (p != null) {
            if (p.getAppliedStereotype("Default::Perspective") != null) {
                validated = false;
                ModelUtil.addMarker(mRsrc, "Class " + cl.getName() + " is located in a <<perspective>> package (" + p.getName() + "). To correct this, move the class to a different package in the model.", IMarker.SEVERITY_ERROR, cl);
                break;
            }
            p = p.getNestingPackage();
        }      
    }

    private Jsp findMatchingJSP(String fromQualifiedName) {
        for (Iterator i = app.getAppJSPs().iterator(); i.hasNext();) {
            Jsp jsp = (Jsp) i.next();
            if (jsp.getOriginalQualifiedName().equals(fromQualifiedName)) {
                return jsp;
            }
        }
        return null;
    }
}
