/*
 * Created on Sep 16, 2005
 */
package com.ibm.jps.uxtoweb.utility;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

/**
 * @author jpschoch
 */
public class JavaUtil {

    /**
     * Given an input string of the type "com.company.project.AClass", getPath will return
     * "com.company.project" (or an empty string if the input string is "AClass").
     * 
     * @param name
     * @return
     */
    public static String getPath(String name) {
        int pos = name.lastIndexOf(".");
        if (pos > 0)
            return name.substring(0, pos);
        else
            return "";
    }

    public static String getPath(String name, char fileSeparator) {
        int pos = name.lastIndexOf(fileSeparator);
        if (pos > 0)
            return name.substring(0, pos);
        else
            return "";
    }

    /**
     * Given an input string of the type "com.company.project.AClass", getSimpleName will return
     * "AClass" (or the string itself if the input string is "AClass").
     * 
     * @param name
     * @return
     */
    public static String getSimpleName(String name) {
        int pos = name.lastIndexOf(".");
        if (pos > 0)
            return name.substring(pos + 1);
        else
            return name;
    }

    public static String getSimpleName(String name, char fileSeparator) {
        int pos = name.lastIndexOf(fileSeparator);
        if (pos > 0)
            return name.substring(pos + 1);
        else
            return name;
    }

    /**
     * @param name
     * @return
     */
    public static String firstCharToUpperCase(String name) {
        return name.substring(0, 1).toUpperCase() + name.substring(1);
    }

    /**
     * @param name
     * @return
     */
    public static String firstCharToLowerCase(String name) {
        return name.substring(0, 1).toLowerCase() + name.substring(1);
    }

    public static String replaceAllSubstrings(String inputString, String fromExp, String toExp) {
        if (fromExp.equals(inputString)) return toExp;
        int pos;
        int prev = 0;
        String res = "";
        do {
            pos = inputString.indexOf(fromExp, prev);
            if (pos == -1) {
                res = res + inputString.substring(prev);
            } else {
                if (pos > prev) res = res + inputString.substring(prev, pos);
                res = res + toExp;
            }
            prev = pos + fromExp.length();
        } while (pos >= 0);
        return res;
    }

    public static String convertToLabel(String inputString) {
         return convertToTitle(inputString) + ":";
    }
    
    public static String convertToTitle(String inputString) {
        String outputString = "";
        int cc = inputString.length();
        for (int i = 0; i < cc; i++) {
            char c = inputString.charAt(i);
            boolean eos = true;
            char cn = 0;
            if (i < cc - 1) {
                eos = false;
                cn = inputString.charAt(i + 1);
            }
            if (i == 0)
                outputString = outputString + Character.toUpperCase(c);
            else {
                if (Character.isUpperCase(c) && !eos && !Character.isUpperCase(cn) && cn != ' '
                        && !outputString.endsWith(" "))
                    outputString = outputString + " " + c;
                else
                    outputString = outputString + c;
            }
        }
        return outputString;
    }
    

    public static List convertToList(String inputString) {
        return convertToList(inputString, ",");
    }
    
    public static List convertToList(String inputString, String sep) {
        List<String> result = new ArrayList<String>();
        int pos;
        int prev = 0;
        do {
            pos = inputString.indexOf(sep, prev);
            if (pos == -1) {
                result.add(inputString.substring(prev).trim());
            } else {
                if (pos > prev) result.add(inputString.substring(prev, pos).trim());
            }
            prev = pos + sep.length();
        } while (pos >= 0);
        return result;
    }
    
    public static void addToList(List<Object> list, Object obj) {
        if (!list.contains(obj)) {
            list.add(obj);
        }
    }
    
	public static String getNameWithoutModel(String name) {
		int pos = name.indexOf("::");
		if (pos > 0)
			return name.substring(name.indexOf("::") + 2);
		else
			return name;
	}

	public static String convertToFullJavaName(String input) {
		return replaceAllSubstrings(getNameWithoutModel(input), "::",
				".");
	}

	public static String convertToSimpleJavaName(String input) {
		return getSimpleName(convertToFullJavaName(input));
	}

	public static String convertToJavaPath(String input) {
		return getPath(convertToFullJavaName(input));
	}

	public static String getQualifiedName(NamedElement elt,
			NamedElement parent, Stereotype st) {
		String fieldType = (String) elt.getValue(st, "path");
		if (fieldType == null || fieldType.equals("")) {
			return replaceAllSubstrings(getNameWithoutModel(parent.getQualifiedName()), "::", ".");
		} else
			return (fieldType + "." + elt.getName());
	}

	public static String getQualifiedName(NamedElement elt, Stereotype st) {
		String fieldType = (String) elt.getValue(st, "path");
		if (fieldType == null || fieldType.equals("")) {
			return replaceAllSubstrings(getNameWithoutModel(elt.getQualifiedName()), "::", ".");
		} else
			return (fieldType + "." + elt.getName());
	}
	
	public static String getDocumentation(Element from) {
		for (Iterator i = from.getOwnedComments().iterator(); i.hasNext();) {
			Comment c = (Comment) i.next();
			if (c.getAppliedStereotype("Default::Documentation") != null) {
				return c.getBody();
			}
		}
		return "";
	}
	
	public static Property findAttribute(Class fromClass, String attribName) {
		for (Iterator i = fromClass.getAttributes().iterator(); i.hasNext();) {
			Property att = (Property) i.next();
			if (attribName.equals(att.getName())) {
				return att;
			}
		}
		return null;
	}
	
	public static Property findPKAttribute(Class fromClass) {
		for (Iterator i = fromClass.getAttributes().iterator(); i.hasNext();) {
			Property att = (Property) i.next();
			if (att.getAppliedStereotype("DataModeling::pk") != null) {
				return att;
			}
		}
		return null;
	}

}
