/*
 * Created on Sep 14, 2005
 */
package com.ibm.jps.uxtoweb.utility;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmf.runtime.common.ui.util.WorkbenchPartActivator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.part.ISetSelectionTarget;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;

import com.ibm.jps.uxtoweb.metamodel.Bean;
import com.ibm.jps.uxtoweb.metamodel.Field;
import com.ibm.jps.uxtoweb.metamodel.Forward;
import com.ibm.jps.uxtoweb.metamodel.ItemList;
import com.ibm.jps.uxtoweb.metamodel.Link;
import com.ibm.jps.uxtoweb.metamodel.ListBox;
import com.ibm.jps.uxtoweb.metamodel.Method;
import com.ibm.jps.uxtoweb.metamodel.RadioButton;
import com.ibm.jps.uxtoweb.metamodel.TextField;
import com.ibm.jps.uxtoweb.metamodel.UserAction;
import com.ibm.jps.uxtoweb.metamodel.UserActionKind;

/**
 * @author jpschoch
 */
public class ModelUtil {
	private static Class foundClass = null;

	private static Association foundAssoc = null;

	private static String foundRole = "";

	public static UserAction populateAction(Class cl, Operation op,
			Stereotype opst, IResource errorRsrc)
			throws InvalidUXModelException {
		UserAction action = new UserAction();

		// UserAction.label
		String label = (String) op.getValue(opst, "label");
		if (label == null || label.equals(""))
			label = JavaUtil.convertToTitle(op.getName());
		action.setLabel(label);

		// UserAction.scope: the attribute's default value is used

		// UserAction kind
		EnumerationLiteral kind = (EnumerationLiteral) op
				.getValue(opst, "kind");
		if (kind.getName().equals("Button"))
			action.setKind(UserActionKind.BUTTON);
		else if (kind.getName().equals("Link"))
			action.setKind(UserActionKind.LINK);
		else if (kind.getName().equals("Reset"))
			action.setKind(UserActionKind.RESET);

		// UserAction.name
		action.setName(JavaUtil.firstCharToLowerCase(op.getName()));

		// UserAction.qualifiedName
		action.setQualifiedName(JavaUtil.getQualifiedName(op, cl, opst));

		// UserAction.jspName
		action.setJspName("/" + JavaUtil.firstCharToLowerCase(cl.getName())
				+ ".jsp");

		// UserAction.forwards
		// Find all associations named "operation" or "operation [condition]".
		// For instance we could
		// have a 'logon' operation/user action and two associations named
		// 'logon [success]' and
		// 'logon [failure]'.
		action.setForwards(findForwards(cl, op.getName()));

		// UserAction.bean
		// Look for a possible input form:
		// - an input form is defined by a composition from the
		// screen to an <<input>> class
		// - if there are multiple actions, the composition must
		// have the name of the action
		int numUserOp = 0;
		for (Iterator i = cl.getOwnedOperations().iterator(); i.hasNext();) {
			Operation oper = (Operation) i.next();
			if (oper.getAppliedStereotype("UXModeling2::useraction") != null)
				numUserOp++;
		}
		AssociatedClass ac;
		if (numUserOp > 1)
			ac = ModelUtil.findCompositionClass(cl, "UXModeling2::input", op
					.getName());
		else
			ac = ModelUtil.findCompositionClass(cl, "UXModeling2::input");
		Association assoc = ac.getAssoc();
		Class inputClass = ac.getAssocClass();

		if (assoc != null && inputClass != null) {
			List<Field> fields = new ArrayList<Field>();
			String beanName = addFields(fields, inputClass, true, errorRsrc);
			// TODO currently only handles one form!
			if (!beanName.equals("")) {
				// TODO scope not handled (defaults to "request")
				Bean bean = new Bean(beanName);
				bean.setFields(fields);
				action.setBean(bean);
			}
		}

		// UserAction.method
		// Look for a bean with an operation matching the useraction.
		action.setMethod(findMethod(cl, op.getName()));

		// UserAction.visibleOnAction
		String s = (String) op.getValue(opst, "visibleOnAction");
		if (s == null) s = "";
		action.setVisibleOnAction(s);

		return action;
	}

	public static ItemList populateList(Class forClass, IResource errorRsrc)
			throws InvalidUXModelException {
		ItemList list = new ItemList();
		List<Field> items = new ArrayList<Field>();
		AssociatedClass ac = ModelUtil.findCompositionClass(forClass,
				"UXModeling2::list");
		Association assoc = ac.getAssoc();
		Class listClass = ac.getAssocClass();
		if (assoc != null && listClass != null) {

			// A <<list>> class exists: retrieve the fields that constitute one
			// element of the list
			String beanName = ModelUtil.addFields(items, listClass, false,
					errorRsrc);

			// Verify that the fields are associated with beans that have a
			// <<pk>> element
			// TODO Should update FacesController JET template to hanlde beans
			// without <<pk>>
			// element
			// TODO Should handle the situation of multiple keys
			String lastBeanChecked = "";
			for (Iterator i = items.iterator(); i.hasNext();) {
				Field f = (Field) i.next();
				Classifier bean = f.getBean();
				if (bean != null) {
					if (!bean.getQualifiedName().equals(lastBeanChecked)) {
						boolean keyFound = false;
						for (Iterator j = bean.getAttributes().iterator(); j
								.hasNext();) {
							Property att = (Property) j.next();
							if (att.getAppliedStereotype("DataModeling::pk") != null) {
								keyFound = true;
								lastBeanChecked = bean.getQualifiedName();
								break;
							}
						}
						if (!keyFound) {
							addMarker(
									errorRsrc,
									"Bean "
											+ bean.getName()
											+ " is used in list "
											+ ac.getRoleName()
											+ ":"
											+ listClass.getName()
											+ ". It should have a primary key defined. "
											+ "To correct this, add the stereotype <<pk>> to the appropriate attribute of the bean.",
									IMarker.SEVERITY_ERROR, bean);
							throw new InvalidUXModelException();
						}
					}
				}
			}

			// The name of the list is derived from the bean of listClass and
			// not from the role name
			// associated with the composition from forClass to listClass. This
			// is necessary
			// because each bean has a unique bean manager but could be used in
			// multiple lists in
			// different forms each with a different name.
			// TODO this is probably a very restrictive way to process lists.
			// How for instance could
			// we handle multiple lists within the same form?
			list.setName(JavaUtil.firstCharToLowerCase(JavaUtil
					.getSimpleName(beanName))
					+ "s");

			// The list of fields should not be empty.
			if (items.isEmpty()) {
				addMarker(
						errorRsrc,
						"<<list>> "
								+ listClass.getName()
								+ " does not contain any valid elements. "
								+ "To correct this, add at least one <<display>> attribute.",
						IMarker.SEVERITY_ERROR, listClass);
				throw new InvalidUXModelException();
			}

			// The fields in the list should be of the same type (a list of CDs
			// should contain only
			// CD items).
			Field firstField = (Field) items.get(0);
			String fieldType = firstField.getBean().getName();
			for (Iterator i = items.iterator(); i.hasNext();) {
				Field f = (Field) i.next();
				if (!f.getBean().getName().equals(fieldType)) {
					addMarker(errorRsrc, "<<list>> " + listClass.getName()
							+ " elements belonging to different entities: "
							+ firstField.getName() + " belongs to " + fieldType
							+ " and " + f.getName() + " to "
							+ f.getBean().getName() + ".",
							IMarker.SEVERITY_ERROR, listClass);
					throw new InvalidUXModelException();
				}
			}

			// rowName corresponds to one element in the list. For instance CD
			// for a list of CDs.
			list.setRowName(fieldType);

			// Finalize
			list.setFields(items);
			return list;
		}
		return null;
	}

	public static Link populateLink(Class forClass,
			NamedElement forClassElement, Stereotype forStereotype,
			IResource errorRsrc) throws InvalidUXModelException {
		// Find the name of the target screen associated with this class through
		// an association
		Class tcl = ModelUtil.findAssocClass(forClass, "UXModeling2::screen",
				AggregationKind.get(AggregationKind.NONE), forClassElement
						.getName());
		if (tcl == null) {
			ModelUtil
					.addMarker(
							errorRsrc,
							"Cannot find target class for <<link>> "
									+ forClassElement.getName()
									+ " in class "
									+ forClass.getName()
									+ ". To correct this, add a directed association from the latter to the <<screen>> class the link should point to.",
							IMarker.SEVERITY_ERROR, forClass);
			throw new InvalidUXModelException();
		}

		Link link = new Link();

		// Link.targetJSPQualifiedName
		link.setTargetJSPQualifiedName(tcl.getQualifiedName());

		// Link.name
		link.setName(forClassElement.getName());

		// Link.label
		String label = (String) forClassElement
				.getValue(forStereotype, "label");
		if (label == null || label.length() == 0)
			label = JavaUtil.convertToTitle(forClassElement.getName());
		link.setLabel(label);

		// Link.text
		link.setText((String) forClassElement.getValue(forStereotype,
				"description"));
		return link;
	}

	public static List findForwards(Class forClass, String refName) {
		List associations = ModelUtil.findAssociations(forClass, refName);
		List<Forward> forwards = new ArrayList<Forward>();
		for (Iterator i = associations.iterator(); i.hasNext();) {
			AssociatedClass acl = (AssociatedClass) i.next();
			Class targetClass = acl.getAssocClass();
			String assocName = acl.getAssoc().getName();
			String forwardName = "success";
			int pos1 = assocName.indexOf('[');
			if (pos1 > 0) {
				int pos2 = assocName.indexOf(']');
				if (pos2 > 0)
					forwardName = assocName.substring(pos1 + 1, pos2);
			}
			// The target class must be a screen
			Stereotype targetScreenStereotype = targetClass
					.getAppliedStereotype("UXModeling2::screen");
			if (targetScreenStereotype != null) {
				forwards.add(new Forward(forwardName, "/"
						+ JavaUtil.firstCharToLowerCase(targetClass.getName())
						+ ".jsp"));
			}
		}
		return forwards;
	}

	public static List findGlobalForwards(Class forClass) {
		return findForwards(forClass, "");
	}

	private static List<AssociatedClass> findAssociations(Class fromClass,
			String refName) {
		List<AssociatedClass> assocs = new ArrayList<AssociatedClass>();
		Package pkg = fromClass.getModel();
		if (pkg != null) {
			addAssociations(assocs, pkg, fromClass, refName);
		}
		return assocs;
	}

	private static void addAssociations(List<AssociatedClass> toList,
			Package inPkg, Class fromClass, String refName) {
		for (Iterator i = inPkg.getOwnedElements().iterator(); i.hasNext();) {
			Element ownedElt = (Element) i.next();
			if (ownedElt instanceof Association) {
				Association assoc = (Association) ownedElt;
				EList members = assoc.getMemberEnds();
				if (members.size() >= 2) {
					Property prop1 = (Property) members.get(0);
					Property prop2 = (Property) members.get(1);
					Type type1 = prop1.getType();
					Type type2 = prop2.getType();
					if (type1 instanceof Class && type2 instanceof Class) {
						Class cl1 = (Class) type1;
						Class cl2 = (Class) type2;
						if (cl1.equals(fromClass) || cl2.equals(fromClass)) {
							boolean addIt = false;
							String assocName = assoc.getName();
							if (assocName != null) {
								if (refName.equals("")) {
									if (assocName.startsWith("["))
										addIt = true;
								} else {
									if (assocName.equals(refName)
											|| assocName.startsWith(refName
													+ " [")) {
										addIt = true;
									} else {
										String s = assocName;
										int pos = assocName.indexOf("[");
										if (pos > 0) {
											s = assocName.substring(0, pos);
										}
										s = s.trim();
										if (s.startsWith(refName + ",")
												|| s.endsWith(", " + refName)
												|| s.indexOf(", " + refName
														+ ",") > 0) {
											addIt = true;
										}
									}
								}
							}
							if (addIt) {
								Class targetClass = (cl1.equals(fromClass)) ? cl2
										: cl1;
								toList.add(new AssociatedClass(targetClass,
										assoc));
							}
						}
					}
				}
			}
		}
		// Search the nested packages
		for (Iterator i = inPkg.getNestedPackages().iterator(); i.hasNext();) {
			addAssociations(toList, (Package) i.next(), fromClass, refName);
		}
	}

	public static AssociatedClass findCompositionClass(Class fromClass,
			String targetClassStereotype, String assocName) {
		findIt(fromClass, targetClassStereotype, AggregationKind
				.get(AggregationKind.COMPOSITE), assocName);
		return new AssociatedClass(foundClass, foundAssoc);
	}

	public static AssociatedClass findCompositionClass(Class fromClass,
			String targetClassStereotype) {
		findIt(fromClass, targetClassStereotype, AggregationKind
				.get(AggregationKind.COMPOSITE), null);
		return new AssociatedClass(foundClass, foundAssoc, foundRole);
	}

	public static Class findAssocClass(Class fromClass,
			String targetClassStereotype, AggregationKind aKind) {
		findIt(fromClass, targetClassStereotype, aKind, null);
		return foundClass;
	}

	public static Class findAssocClass(Class fromClass,
			String targetClassStereotype, AggregationKind aKind,
			String assocName) {
		findIt(fromClass, targetClassStereotype, aKind, assocName);
		return foundClass;
	}

	/**
	 * findMatchingColumn cycles through the dependencies of the specified
	 * property. If the other end of the dependency is a property belonging to a <
	 * <table>>class, then it is the matching column.
	 */
	public static void setMatchingColumn(Property fromAtt, Field targetField) {
		for (Iterator i = fromAtt.getClientDependencies().iterator(); i
				.hasNext();) {
			Dependency dep = (Dependency) i.next();
			for (Iterator j = dep.getSuppliers().iterator(); j.hasNext();) {
				NamedElement elt = (NamedElement) j.next();
				if (elt instanceof Property) {
					Property att = (Property) elt;
					Namespace ns = att.getNamespace();
					if (ns != null && ns instanceof Class) {
						Class cl = (Class) ns;
						if (cl.getAppliedStereotype("DataModeling::table") != null) {
							targetField.setMappedToColumn(att.getName());
							targetField.setMappedToTable(cl.getName());
						}
					}
				}
			}
		}
	}

	/**
	 * fromClass is typically an < <input>>or < <list>>entity. For the attribute
	 * identified by its name (attribName parameter), findAttribClass identifies
	 * the < <bean>>class containing a similarly named attribute. The <
	 * <bean>>class is selected among the classes fromClass is point to through
	 * dependency relationships.
	 * 
	 * @param fromClass
	 * @param attribName
	 * @return
	 */

	public static FieldClass findAttribClass(Class fromClass, String attribName) {
		FieldClass fc = new FieldClass();
		for (Iterator i = fromClass.getClientDependencies().iterator(); i
				.hasNext();) {
			Dependency dep = (Dependency) i.next();
			for (Iterator j = dep.getSuppliers().iterator(); j.hasNext();) {
				NamedElement elt = (NamedElement) j.next();
				if (elt instanceof Class) {
					Class supplier = (Class) elt;
					if (supplier.getAppliedStereotype("UXModeling2::bean") != null) {
						for (Iterator k = supplier.getAttributes().iterator(); k
								.hasNext();) {
							Property att = (Property) k.next();
							if (att.getName().equals(attribName)) {
								fc.setFieldClass(supplier);
								fc.setFieldAtt(att);
								return fc;
							}
						}
					}
				}
			}
		}
		return fc;
	}

	/**
	 * 
	 * @param fromClass
	 * @param methodName
	 * @return
	 */
	public static Method findMethod(Class fromClass, String methodName) {
		for (Iterator i = fromClass.getClientDependencies().iterator(); i
				.hasNext();) {
			Dependency dep = (Dependency) i.next();
			for (Iterator j = dep.getSuppliers().iterator(); j.hasNext();) {
				NamedElement elt = (NamedElement) j.next();
				if (elt instanceof Class) {
					Class supplier = (Class) elt;
					Method m = createMethod(supplier, methodName,
							"UXModeling2::bean");
					if (m != null)
						return m;
				}
			}
		}
		return null;
	}

	/**
	 * @param supplier
	 * @param methodName
	 * @return
	 */
	private static Method createMethod(Class supplier, String methodName,
			String stereotypeName) {
		Stereotype st = supplier.getAppliedStereotype(stereotypeName);
		if (st != null) {
			for (Iterator i = supplier.getOwnedOperations().iterator(); i
					.hasNext();) {
				Operation sop = (Operation) i.next();
				if (sop.getName().equals(methodName)) {
					Method m = new Method();
					m.setUmlBean(supplier);
					m.setName(methodName);
					m.setBeanQualifiedName(JavaUtil.getQualifiedName(supplier, st));
					return m;
				}
			}
		}
		return null;
	}

	/**
	 * Finds the association named 'assocName' of type specified by 'aKind' for
	 * class 'fromClass'
	 * 
	 * @param fromClass
	 *            the origin of the association
	 * @param targetClassStereotype
	 *            the stereotype of the target class (if specified)
	 * @param aKind
	 *            the type of the association to find (aggregation, composition
	 *            or straight association) (if specified)
	 * @param assocName
	 *            the name of the association to find (if specified)
	 */
	private static void findIt(Class fromClass, String targetClassStereotype,
			AggregationKind aKind, String assocName) {
		foundAssoc = null;
		foundClass = null;
		// Search the package of the specified class
		Package pkg = fromClass.getPackage();
		if (pkg != null) {
			searchPackage(pkg, fromClass, targetClassStereotype, aKind,
					assocName);
		}
		// Search the whole model if not found
		if (foundClass == null) {
			Model model = fromClass.getModel();
			searchPackage(model, fromClass, targetClassStereotype, aKind,
					assocName);
		}
	}

	/**
	 * @param pkg
	 * @param fromClass
	 * @param targetClassStereotype
	 * @param kind
	 * @param assocName
	 */
	private static void searchPackage(Package pkg, Class fromClass,
			String targetClassStereotype, AggregationKind aKind,
			String assocName) {
		for (Iterator i = pkg.getOwnedElements().iterator(); i.hasNext();) {
			Element ownedElt = (Element) i.next();
			if (ownedElt instanceof Association) {
				Association assoc = (Association) ownedElt;
				EList members = assoc.getMemberEnds();
				if (members.size() >= 2) {
					Property prop1 = (Property) members.get(0);
					Property prop2 = (Property) members.get(1);
					Type type1 = prop1.getType();
					Type type2 = prop2.getType();
					if (type1 instanceof Class && type2 instanceof Class) {
						Class cl1 = (Class) type1;
						Class cl2 = (Class) type2;
						Class targetClass = null;
						Property prop = null;
						if (cl1.equals(fromClass)) {
							targetClass = cl2;
							prop = prop2;
						} else if (cl2.equals(fromClass)) {
							targetClass = cl1;
							prop = prop1;
						}
						// Only process further if one of the ends matches
						// fromClass and if the
						// targetClass is navigable (non-navigable ends are
						// owned by the association
						// itself instead of being owned by the class)
						if (prop != null
								&& !assoc.getOwnedEnds().contains(prop)) {
							if (aKind == null
									|| prop.getAggregation().equals(aKind)) {
								if (assocName == null || assocName.equals("")
										|| assocName.equals(assoc.getName())) {
									foundAssoc = assoc;
									if (targetClassStereotype == null
											|| targetClassStereotype.equals("")
											|| targetClass
													.getAppliedStereotype(targetClassStereotype) != null) {
										foundClass = targetClass;
										foundRole = prop.getName();
										return;
									}
								}
							}
						}
					}
				}
			}
		}
		// Search the package nested packages
		for (Iterator i = pkg.getNestedPackages().iterator(); i.hasNext();) {
			searchPackage((Package) i.next(), fromClass, targetClassStereotype,
					aKind, assocName);
			if (foundClass != null)
				return;
		}
	}


	/**
	 * @param toList
	 * @param cl
	 * @param isInputForm
	 * @param errorRsrc
	 * @return - the qualified name of the bean the list is referring to
	 * @throws InvalidUXModelException
	 */
	public static String addFields(List<Field> toList, Class cl,
			boolean isInputForm, IResource errorRsrc)
			throws InvalidUXModelException {
		String refBeanQualifiedName = "";
		for (Iterator i = cl.getAttributes().iterator(); i.hasNext();) {
			Property att = (Property) i.next();
			for (Iterator j = att.getAppliedStereotypes().iterator(); j
					.hasNext();) {
				Stereotype attst = (Stereotype) j.next();
				String stName = attst.getQualifiedName();
				boolean createIt = false;
				// Display fields only
				if (!isInputForm) {
					if (stName.equals("UXModeling2::display"))
						createIt = true;
				} else {
					if (stName.equals("UXModeling2::inputtext")
							|| stName.equals("UXModeling2::listbox")
							|| stName.equals("UXModeling2::radiobutton"))
						createIt = true;
				}
				if (createIt) {
					Field field = createField(cl, att, attst, errorRsrc);
					if (field != null) {
						// All valid fields in the list must be from the same
						// bean.
						if (!refBeanQualifiedName.equals("")
								&& !refBeanQualifiedName.equals(field
										.getBeanQualifiedName())) {
							System.out
									.println("!!! Attribute "
											+ field.getName()
											+ " doesn't belong to the same bean as other attributes.");
							ModelUtil
									.addMarker(
											errorRsrc,
											"Attribute "
													+ field.getName()
													+ " in class "
													+ cl.getName()
													+ " has been ignored as it refers to <<bean>> "
													+ field.getBean().getName()
													+ ", which is different from the bean referenced by previously processed attributes. To correct this, make sure that all attributes"
													+ " refer to the same <<bean>>.",
											IMarker.SEVERITY_WARNING, cl);
						} else {
							refBeanQualifiedName = field.getBeanQualifiedName();
							if (field instanceof TextField) {
								String s = (String) att.getValue(attst, "max");
								if (s == null)
									s = "";
								((TextField) field).setMax(s);
								s = (String) att.getValue(attst, "min");
								if (s == null)
									s = "";
								((TextField) field).setMin(s);
								((TextField) field).setRequired(((Boolean) att
										.getValue(attst, "required"))
										.booleanValue());
								((TextField) field).setPassword(((Boolean) att
										.getValue(attst, "password"))
										.booleanValue());
							}
							Stereotype st = att
									.getAppliedStereotype("UXModeling2::link");
							if (st != null) {
								Link link = ModelUtil.populateLink(cl, att, st,
										errorRsrc);
								field.setLink(link);
							}
							st = att.getAppliedStereotype("DataModeling::pk");
							if (st != null) {
								field.setPrimaryKey(true);
							}
							toList.add(field);
						}
					}
				}
			}
		}
		return refBeanQualifiedName;
	}

	/**
	 * @param cl
	 * @param att
	 */
	private static Field createField(Class cl, Property att, Stereotype attst,
			IResource errorRsrc) {
		String fieldName = att.getName();

		// Retrieve the <<bean>> class containing a matching fieldName
		FieldClass fClass = findAttribClass(cl, fieldName);
		Class attParent = fClass.getFieldClass();

		if (attParent == null) {
			System.out.println("!!! Attribute " + fieldName
					+ " doesn't have a parent class: attribute ignored");
			ModelUtil
					.addMarker(
							errorRsrc,
							"Attribute "
									+ fieldName
									+ " in "
									+ cl.getName()
									+ " doesn't have a parent class: attribute ignored. To correct this, add a dependency from "
									+ cl.getName()
									+ " to the <<bean>> class containing "
									+ fieldName + ".",
							IMarker.SEVERITY_WARNING, att);
			return null;
		} else {
			String beanQualifiedName = JavaUtil.getQualifiedName(attParent, attParent
					.getAppliedStereotype("UXModeling2::bean"));
			Field field = null;
			if (attst.getQualifiedName().equals("UXModeling2::display")) {
				field = new Field(fieldName, beanQualifiedName);
			} else if (attst.getQualifiedName()
					.equals("UXModeling2::inputtext")) {
				field = new TextField(fieldName, beanQualifiedName);
			} else if (attst.getQualifiedName().equals("UXModeling2::listbox")) {
				field = new ListBox(fieldName, beanQualifiedName);
			} else if (attst.getQualifiedName().equals(
					"UXModeling2::radiobutton")) {
				field = new RadioButton(fieldName, beanQualifiedName);
			}
			field.setBean(attParent);
			field.setBeanAtt(fClass.getFieldAtt());
			String fieldLabel = (String) att.getValue(attst, "label");
			if (fieldLabel == null || fieldLabel.equals(""))
				fieldLabel = JavaUtil.convertToTitle(att.getName());
			field.setLabel(fieldLabel);

			// Check the Options stereotype.
			checkOptions(field, att, errorRsrc);

			return field;
		}
	}

	/**
	 * @param field
	 * @param att
	 */
	public static void checkOptions(Field field, Property att,
			IResource errorRsrc) {
		Stereotype opst = att.getAppliedStereotype("UXModeling2::options");
		if (opst != null) {
			// Option "sortable": currently only Strings are supported. If this
			// is not the case
			// generate a warning and ignore the setting.
			boolean b = ((Boolean) att.getValue(opst, "sortable"))
					.booleanValue();
			if (b) {
				if (!field.getBeanAtt().getType().getName().equals("String")) {
					ModelUtil.addMarker(errorRsrc,
							"In the current version, sortable elements must be of type String. "
									+ "The 'sortable' option for attribute "
									+ field.getName() + " (in "
									+ field.getBean().getName()
									+ ") will be ignored.",
							IMarker.SEVERITY_WARNING, field.getBean());
				} else {
					field.setSortable(b);
				}
			}
		}
	}

	public static IProject getProject(NamedElement elt) {
		Model model = elt.getModel();
		Resource r = model.eResource();
		if (r != null) {
			URI uri = r.getURI();
			IWorkspaceRoot wr = ResourcesPlugin.getWorkspace().getRoot();
			String pName = uri.path().substring("/resource".length() + 1);
			int pos = pName.indexOf("/");
			if (pos > 0) {
				pName = pName.substring(0, pos);
				return wr.getProject(pName);
			}
		}
		return null;
	}



	public static IResource getModelResource(NamedElement elt) {
		Resource r = elt.eResource();
		if (r == null) {
			r = elt.getModel().eResource();
		}
		if (r != null) {
			URI uri = r.getURI();
			IWorkspaceRoot wr = ResourcesPlugin.getWorkspace().getRoot();
			String pName = uri.path().substring("/resource".length() + 1);
			String mName = null;
			int pos = pName.indexOf("/");
			if (pos > 0) {
				mName = pName.substring(pos + 1);
				pName = pName.substring(0, pos);
				IProject prj = wr.getProject(pName);
				if (prj != null) {
					return prj.findMember(mName);
				}
			}
		}
		return null;
	}

	public static IMarker addMarker(IResource onRsrc, String description,
			int severity, NamedElement problem) {
		System.err.println(description);
		if (onRsrc == null)
			return null;
		try {
			IMarker marker = onRsrc
					.createMarker("com.ibm.jps.uxtoweb.uxToWebMarker");
			System.out
					.println("Creating marker com.ibm.jps.uxtoweb.uxToWebMarker for "
							+ problem.eClass().getName()
							+ " "
							+ problem.getQualifiedName());
			if (marker.exists()) {
				HashMap<String, Object> attribMap = new HashMap<String, Object>();
				attribMap.put(IMarker.MESSAGE, description);
				attribMap.put(IMarker.PRIORITY, new Integer(
						IMarker.PRIORITY_NORMAL));
				attribMap.put(IMarker.SEVERITY, new Integer(severity));
				attribMap.put(IMarker.DONE, Boolean.FALSE);
				marker.setAttributes(attribMap);
				return marker;
			}
		} catch (CoreException e) {
			System.err.println("Marker does not exist.");
		}
		return null;
	}

	/**
	 * @param o
	 * 
	 * Selects a model element in the Model Explorer
	 * 
	 */
	public static void selectInModelExplorer(Object o) {
		if (o != null && o instanceof EModelElement) {
			ArrayList<EModelElement> elements = new ArrayList<EModelElement>();
			elements.add((EModelElement) o);
			selectInModelExplorer(elements);
		}
	}

	/**
	 * @param o
	 * 
	 * Selects a list of elements in the Model Explorer
	 * 
	 */
	public static void selectInModelExplorer(List list) {
		if (list != null) {
			IViewPart view = WorkbenchPartActivator
					.showView("com.ibm.xtools.modeler.ui.views.modelexplorer.ModelExplorer");
			if (view != null && (view instanceof ISetSelectionTarget)) {
				ISelection selection = new StructuredSelection(list);
				((ISetSelectionTarget) view).selectReveal(selection);
			}
		}
	}
}
